==> hermes-6.0.1.46.tar.gz: hermes-6.0.1.46/lib/hermes/rex_client/tasks/update_structure_task.rb <==
module Hermes
  module RexClient
    class UpdateStructureTask

      def update_entity(rex_entity, className, client = Hermes::RexClient::Client.new, dont_update_parent = true,  arbo_ids = [], already_updated_parent = false)
        rex_entity.stringify_keys!
        fields = rex_entity['fields'].stringify_keys
        return if fields['id_cristal'].blank?

        client.log_puts("Struture fields #{fields}")
        
        arbo_ids << fields['id_cristal']
        structure = className.constantize.find_by(id_cristal: fields['id_cristal'])
        
        if !structure
          client.log_puts("Creating new structure #{fields['id_cristal']}")
          structure = className.constantize.create(generate_structure_fields(fields, rex_entity['id']))
          client.log_puts("End Creating new structure #{fields['id_cristal']}")
        else
          client.log_puts("Updating existant structure #{fields['id_cristal']} #{structure.id}")
          structure.set(generate_structure_fields(fields, rex_entity['id']).merge(updated_at: DateTime.now))
          client.log_puts("End Updating existant structure #{fields['id_cristal']} #{structure.id}")
        end
        
        if fields['id_parent_cristal'].blank? 
          structure.parent_id = nil 
        else
          if !arbo_ids.include?(fields['id_parent_cristal'])
            parent = className.constantize.find_by(id_cristal: fields['id_parent_cristal'])
            if !already_updated_parent && !parent
              client.log_puts("Updating parent structure #{fields['id_parent_cristal']}")
              parent_rex_entity = client.search_structures({"fields.id_cristal": "|#{fields['id_parent_cristal']}"}).first
              client.log_puts("Found parent rex")if parent_rex_entity
              parent = update_entity(parent_rex_entity, className, client, dont_update_parent, arbo_ids, true) if parent_rex_entity
            end
            structure.parent_id = parent && parent.id 
            client.log_puts("Affected new parent #{structure.parent_id}") 
          end
        end
        
        client.log_puts("Starting save structure: #{structure.id_cristal} changed : #{structure.changes}") 
        structure.save!
        client.log_puts("End save structure: #{structure.id_cristal}") 

        if className == 'Hermes::Entity'
          if fields['itiephycrenational'] != structure.cre_national_id_cristal
            client.log_puts("Start update cre nationnal with id #{fields['itiephycrenational']}")

            current_cre_national = Hermes::UserRight.find_by(type: 'entity', entity_id: structure.id, role: 'cre_national')
            client.log_puts("Found existing cre national user right for this entity #{current_cre_national.id}") if current_cre_national

            if current_cre_national
              still_cre = false 
              
              if structure.cre_national_id_cristal
                contact_rex_json = client.search_contacts({"fields.id_cristal": "|#{structure.cre_national_id_cristal}"}).first
                still_cre = contact_rex_json['fields']['id_etablissement_cristal'].include?(structure.id_cristal)
              end 
              
              if still_cre
                client.log_puts("Downgraing old user right => cre local")
                current_cre_national.update(role: 'cre')
              else
                client.log_puts("Removing old cre national")
                current_cre_national.destroy
              end 
            end  

            if fields['itiephycrenational'].blank?
              client.log_puts("Clearing cre national fields")
              structure.update(
                cre_national_id_cristal: nil,
                cre_national_first_name: nil,
                cre_national_last_name: nil,
                cre_national_email: nil,
              )
            else 

              unless user_cre_national = User.find_by(id_cristal: fields['itiephycrenational'])
                client.log_puts("User not found with id cristal")
                if user_cre_national = User.find_by(email: fields['ladrmailcrenational'], id_cristal: nil) 
                  client.log_puts("User found with email #{user_cre_national.id}")
                  user_cre_national.update(id_cristal: fields['itiephycrenational'])
                else
                  client.log_puts("Creating new user")
                  email = fields['ladrmailcrenational'].downcase
                  user_cre_national = User.create!({
                    username: email,
                    first_name: fields['lprenomcrenational'],
                    last_name: fields['lnomcrenational'],
                    id_cristal: fields['itiephycrenational'],
                    email: email,
                    login: email,
                    auth_roles: ['cre'] ,
                    phone_number: "1234567890",
                    provider: HERMES_CONFIG['identity_provider_key'],
                    create_user_data: {
                      gender: 'M',
                      firstName: fields['lprenomcrenational'],
                      lastName: fields['lnomcrenational'],
                      email: email,
                      login: email,
                      mobileNumber: "1234567890",
                      roles: ['cre']
                    }
                  }.merge(password: Devise.friendly_token[0,20]))
                end 
              end 

              if user_right_local = Hermes::UserRight.find_by(type: 'entity', entity_id: structure.id, user_id: user_cre_national.id)
                client.log_puts("Removing old local user right #{user_right_local}")
                user_right_local.destroy
              end 

              client.log_puts("Creating user right for new cre national")
              Hermes::UserRight.create!(
                type: 'entity', 
                entity_id: structure.id, 
                role: 'cre_national',
                right: 'administrator',
                user_id: user_cre_national.id,
                origin: 'rex',
              )

              client.log_puts("Updating entity cre national fields")
              structure.update(
                cre_national_id_cristal: fields['itiephycrenational'],
                cre_national_first_name: fields['lprenomcrenational'],
                cre_national_last_name: fields['lnomcrenational'],
                cre_national_email: fields['ladrmailcrenational'],
              )
            end

            client.log_puts("End update cre nationnal")
          end 
        end 

        structure
      end

      def generate_structure_fields(fields , id)      
        {
          title: fields['app_comm'],
          id_rex: id,
          id_cristal: fields['id_cristal'],
          id_parent_cristal: fields['id_parent_cristal'],
          siret: fields['siret'].length == 14 ? fields['siret'] : nil,
        }
      end
    end
  end
end

==> hermes-6.0.2.12.tar.gz: hermes-6.0.2.12/lib/hermes/rex_client/tasks/update_structure_task.rb <==
module Hermes
  module RexClient
    class UpdateStructureTask

      def update_entity(rex_entity, className, client = Hermes::RexClient::Client.new, dont_update_parent = true,  arbo_ids = [], already_updated_parent = false)
        rex_entity.stringify_keys!
        fields = rex_entity['fields'].stringify_keys
        return if fields['id_cristal'].blank?

        client.log_puts("Struture fields #{fields}")
        
        arbo_ids << fields['id_cristal']
        structure = className.constantize.find_by(id_cristal: fields['id_cristal'])
        
        if !structure
          client.log_puts("Creating new structure #{fields['id_cristal']}")
          structure = className.constantize.create(generate_structure_fields(fields, rex_entity['id']))
          client.log_puts("End Creating new structure #{fields['id_cristal']}")
        else
          client.log_puts("Updating existant structure #{fields['id_cristal']} #{structure.id}")
          structure.set(generate_structure_fields(fields, rex_entity['id']).merge(updated_at: DateTime.now))
          client.log_puts("End Updating existant structure #{fields['id_cristal']} #{structure.id}")
        end
        
        if fields['id_parent_cristal'].blank? 
          structure.parent_id = nil 
        else
          if !arbo_ids.include?(fields['id_parent_cristal'])
            parent = className.constantize.find_by(id_cristal: fields['id_parent_cristal'])
            if !already_updated_parent && !parent
              client.log_puts("Updating parent structure #{fields['id_parent_cristal']}")
              parent_rex_entity = client.search_structures({"fields.id_cristal": "|#{fields['id_parent_cristal']}"}).first
              client.log_puts("Found parent rex")if parent_rex_entity
              parent = update_entity(parent_rex_entity, className, client, dont_update_parent, arbo_ids, true) if parent_rex_entity
            end
            structure.parent_id = parent && parent.id 
            client.log_puts("Affected new parent #{structure.parent_id}") 
          end
        end
        
        client.log_puts("Starting save structure: #{structure.id_cristal} changed : #{structure.changes}") 
        structure.save!
        client.log_puts("End save structure: #{structure.id_cristal}") 

        if className == 'Hermes::Entity'
          if fields['itiephycrenational'] != structure.cre_national_id_cristal
            client.log_puts("Start update cre nationnal with id #{fields['itiephycrenational']}")

            current_cre_national = Hermes::UserRight.find_by(type: 'entity', entity_id: structure.id, role: 'cre_national')
            client.log_puts("Found existing cre national user right for this entity #{current_cre_national.id}") if current_cre_national

            if current_cre_national
              still_cre = false 
              
              if structure.cre_national_id_cristal
                contact_rex_json = client.search_contacts({"fields.id_cristal": "|#{structure.cre_national_id_cristal}"}).first
                still_cre = !!contact_rex_json && contact_rex_json['fields']['id_etablissement_cristal'].include?(structure.id_cristal)
              end 
              
              if still_cre
                client.log_puts("Downgraing old user right => cre local")
                current_cre_national.update(role: 'cre')
              else
                client.log_puts("Removing old cre national")
                current_cre_national.destroy
              end 
            end  

            if fields['itiephycrenational'].blank?
              client.log_puts("Clearing cre national fields")
              structure.update(
                cre_national_id_cristal: nil,
                cre_national_first_name: nil,
                cre_national_last_name: nil,
                cre_national_email: nil,
              )
            else 

              unless user_cre_national = User.find_by(id_cristal: fields['itiephycrenational'])
                client.log_puts("User not found with id cristal")
                if user_cre_national = User.find_by(email: fields['ladrmailcrenational'], id_cristal: nil) 
                  client.log_puts("User found with email #{user_cre_national.id}")
                  user_cre_national.update(id_cristal: fields['itiephycrenational'])
                else
                  client.log_puts("Creating new user")
                  email = fields['ladrmailcrenational'].downcase
                  user_cre_national = User.create!({
                    username: email,
                    first_name: fields['lprenomcrenational'],
                    last_name: fields['lnomcrenational'],
                    id_cristal: fields['itiephycrenational'],
                    email: email,
                    login: email,
                    auth_roles: ['cre'] ,
                    phone_number: "1234567890",
                    provider: HERMES_CONFIG['identity_provider_key'],
                    create_user_data: {
                      gender: 'M',
                      firstName: fields['lprenomcrenational'],
                      lastName: fields['lnomcrenational'],
                      email: email,
                      login: email,
                      mobileNumber: "1234567890",
                      roles: ['cre']
                    }
                  }.merge(password: Devise.friendly_token[0,20]))
                end 
              end 

              if user_right_local = Hermes::UserRight.find_by(type: 'entity', entity_id: structure.id, user_id: user_cre_national.id)
                client.log_puts("Removing old local user right #{user_right_local}")
                user_right_local.destroy
              end 

              client.log_puts("Creating user right for new cre national")
              Hermes::UserRight.create!(
                type: 'entity', 
                entity_id: structure.id, 
                role: 'cre_national',
                right: 'administrator',
                user_id: user_cre_national.id,
                origin: 'rex',
              )

              client.log_puts("Updating entity cre national fields")
              structure.update(
                cre_national_id_cristal: fields['itiephycrenational'],
                cre_national_first_name: fields['lprenomcrenational'],
                cre_national_last_name: fields['lnomcrenational'],
                cre_national_email: fields['ladrmailcrenational'],
              )
            end

            client.log_puts("End update cre nationnal")
          end 
        end 

        structure
      end

      def generate_structure_fields(fields , id)      
        {
          title: fields['app_comm'],
          id_rex: id,
          id_cristal: fields['id_cristal'],
          id_parent_cristal: fields['id_parent_cristal'],
          siret: fields['siret'].length == 14 ? fields['siret'] : nil,
        }
      end
    end
  end
end
